#!/usr/bin/python2
# -*- coding: utf-8 -*-
'''
This file is part of PyX3 - Huawei X3 Toolkit

    Copyright (C) 2013-2015 ElTonno @ Android-Hilfe.de

    PyX3 is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyX3 is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
'''

__title__ = 'DBridge'
__description__ = 'Custom ADB-Connection-Script'
__version__ = 'rev 20181021'
__author__ = 'ElTonno @ Android-Hilfe.de'
__copyright__ = 'Copyright (C) 2013-2018'

try:
    import sys
    import re
    import time
    from PyQt4.QtGui import qApp
    from PyQt4.QtCore import QProcess
except ImportError as e:
    print "[!] Required module missing. %s" % e.args[0]
    sys.exit(-1)   
    
reload(sys)
sys.setdefaultencoding('UTF8')

# QProcess
qproc = QProcess()

def MyProcess(cmd):
    print cmd
    qproc_output = ''
    qproc.setProcessChannelMode(QProcess.MergedChannels)
    qproc.start(cmd)
    qproc.waitForStarted()
    while qproc.waitForFinished(-1):
        qproc_output += str(qproc.readAll())
        time.sleep(0.002)
        if qproc_output:
            qApp.processEvents()
        if qproc.state() != QProcess.Running:
            qproc.terminate()
            #print ' QProcess: BREAK! '
            break            
    return qproc_output 

def MyProcessWait(cmd):
    qproc.setProcessChannelMode(QProcess.MergedChannels)
    qproc.start(cmd)
    qproc.waitForStarted()
    qproc.waitForFinished()
    return str(qproc.readAll())

def killProcess():
    qproc.terminate()

def terminate():
    qproc.terminate()
    sys.exit(-1)

##  Android Debug Bridge

class Adb:
    def __init__(self, adb='adb', log=''):
        self.adb = adb
        self.log = log
        self.shellActive = False
        self.emuActive = False

    def Exec(self, cmd):
        if self.shellActive:
            output = MyProcess(u'{0} shell {1}'.format(self.adb, cmd))
        elif self.emuActive:
            output = MyProcess(u'{0} emu {1}'.format(self.adb, cmd))
        else:
            output = MyProcess(u'{0} {1}'.format(self.adb, cmd))
        #output = '\n'.join(x for x in [x for x in filter(None, output.splitlines())])
        if self.log == 'cmd':
            print cmd
        elif self.log == 'out':
            print output
        if 'device not found' in output:
            return
        return output

    # Commands
    def _devices(self, value=''):
        return self.Exec('devices {0}'.format(value))

    def _connect(self, host, port='5555'):
        return self.Exec('connect {0}:{1}'.format(host, port))

    def _disconnect(self, host, port='5555'):
        return self.Exec('disconnect {0}:{1}'.format(host, port))

    def _push(self, local, remote):
        return self.Exec('push \"{0}\" \"{1}\"'.format(local, remote))

    def _pull(self, remote, local):
        return self.Exec('pull \"{0}\" \"{1}\"'.format(remote, local))

    def _sync(self, directory=''):
        return self.Exec('sync {0}'.format(directory))

    def _shell(self, cmd):
        self.shellActive = True
        return self.Exec(cmd)

    def _emu(self, cmd):
        self.emuActive = True
        return self.Exec(cmd)

    def _logcat(self, filtern=''):
        return self.Exec('logcat {0}'.format(filtern))

    def _forward(self, local, remote):
        return self.Exec('forward {0} {1}'.format(local, remote))

    def _jdwp(self):
        return self.Exec('jdwp')

    def _install(self, afile, value=''):
        return self.Exec('install {0} \"{1}\"'.format(value, afile))

    def _uninstall(self, package, value=''):
        return self.Exec('uninstall {0} {1}'.format(value, package))

    def _bugreport(self):
        return self.Exec('bugreport')

    def _backup(self, value):
        return self.Exec('backup {0}'.format(value))

    def _restore(self, afile):
        return self.Exec('restore {0}'.format(afile))

    def _help(self):
        return self.Exec('help')

    def _version(self):
        return self.Exec('version')

    # Scripts
    def _wait(self):
        return self.Exec('wait-for-device')

    def _start(self):
        return self.Exec('start-server')

    def _kill(self):
        return self.Exec('kill-server')

    def _state(self):
        return self.Exec('get-state')

    def _serial(self):
        return self.Exec('get-serialno')

    def _devpath(self):
        return self.Exec('get-devpath')

    def _status(self):
        return self.Exec('status-window')

    def _remount(self):
        return self.Exec('remount')

    def _unmount(self, mntpoint):
        self.emuActive = True
        return self.Exec('umount {0}'.format(mntpoint))

    def _reboot(self, value=''):
        return self.Exec('reboot {0}'.format(value))

    def _shutdown(self):
        return self.Exec('reboot -p')

    def _root(self):
        return self.Exec('root')

    def _usb(self):
        return self.Exec('usb')

    def _tcpip(self, port):
        return self.Exec('tcpip {0}'.format(port))

    def _ppp(self, tty, value=''):
        return self.Exec('ppp {0} {1}'.format(tty, value))

    def _ADB_TRACE(self, value='all'):
        return self.Exec('ADB_TRACE {0}'.format(value))

    def _ANDROID_SERIAL(self):
        return self.Exec('ANDROID_SERIAL')

    def _ANDROID_LOG_TAGS(self, value):
        return self.Exec('ANDROID_LOG_TAGS={0}'.format(value))

    def _ANDROID_DATA(self):
        ret = self._shell('\"echo $ANDROID_DATA\"')
        return (ret.strip() if ret else None)

    def _ANDROID_ROOT(self):
        ret = self._shell('\"echo $ANDROID_ROOT\"')
        return (ret.strip() if ret else None)

    def _EXTERNAL_STORAGE(self):
        ret = self._shell('\"echo $EXTERNAL_STORAGE\"')
        return (ret.strip() if ret else None)

    def _SD_EXT_DIRECTORY(self):
        ret = self._shell('\"echo $SD_EXT_DIRECTORY\"')
        return (ret.strip() if ret else None)


## FASTBOOT

class Fastb:

    def __init__(self, fastboot='fastboot'):
        self.fastboot = fastboot

    def Exec(self, cmd):
        output = MyProcess('{0} {1}'.format(self.fastboot, cmd))
        #output = '\n'.join(x for x in [x for x in filter(None, output.splitlines())])
        if 'device not found' in output:
            return        
        return output        

    # Commands
    def _update(self, filename):
        return self.Exec('update {0}'.format(filename))

    def _flashall(self):
        return self.Exec('flashall')

    def _flash(self, partition, filename):
        return self.Exec('flash {0} {1}'.format(partition, filename))

    def _erase(self, partition):
        return self.Exec('erase {0}'.format(partition))

    def _format(self, partition):
        return self.Exec('format {0}'.format(partition))

    def _getvar(self, variable):
        return self.Exec('getvar {0}'.format(variable))

    def _boot(self, kernel, ramdisk=''):
        return self.Exec('boot {0} {1}'.format(kernel, ramdisk))

    def _flashraw(self, kernel, ramdisk=''):
        return self.Exec('flash:raw boot {0} {1}'.format(kernel, ramdisk))

    def _devices(self, value=''):
        return self.Exec('devices {0}'.format(value))

    def _continueboot(self):
        return self.Exec('continue')

    def _reboot(self, value=''):
        return self.Exec('reboot {0}'.format(value))

    def _shutdown(self):
        return self.Exec('oem powerdown')

    def _help(self):
        return self.Exec('help')


##  Android Asset Packaging Tool

class Aapt:

    def __init__(self, aapt='aapt'):
        self.aapt = aapt

    def Exec(self, cmd):
        output = MyProcessWait('{0} {1}'.format(self.aapt, cmd))
        #output = '\n'.join(x for x in [x for x in filter(None, output.splitlines())])
        return output

    def AppData(self, apk):
        if not apk.endswith('.apk'):
            apk+='.apk'
        return self.Exec('dump badging \"{0}\"'.format(apk))
        
    def AppLabel(self, apk):
        if not apk.endswith('.apk'):
            apk+='.apk'
        pkg_re = re.findall('application-label:\'(.*?)\'', self.Exec('dump badging \"{0}\"'.format(apk)))
        return (pkg_re[0] if pkg_re else self.PackageLabel(apk))

    def PackageLabel(self, apk):
        if not apk.endswith('.apk'):
            apk+='.apk'
        pkg_re = re.findall('package: name=\'(.*?)\'', self.Exec('dump badging \"{0}\"'.format(apk)))
        return (' ' if not pkg_re else pkg_re[0])

    def IconPath(self, apk):        
        if not apk.endswith('.apk'):
            apk+='.apk'
        pkg_re = re.findall('application: label=\'.*\'\s+icon=\'(.*)\'', self.Exec('dump badging \"{0}\"'.format(apk))) 
        return (' ' if not pkg_re else pkg_re[0])

    def VersionName(self, apk):        
        if not apk.endswith('.apk'):
            apk+='.apk'
        pkg_re = re.findall('package:.*versionName=\'(.*?)\'', self.Exec('dump badging \"{0}\"'.format(apk)))
        return (' ' if not pkg_re else pkg_re[0])
    
class Custom:

    def __init__(self, adb='adb', fastboot='fastboot'):
        self.adb = adb
        self.fastboot = fastboot

    def _GetAttached(self):
        dev = re.findall(r"(.+)\t(.+)", Adb(self.adb)._devices())
        if dev:     
            return dev[0]
        else:
            dev = re.findall(r"(.+)\t(.+)", Fastb(self.fastboot)._devices())
            return (False if not dev else dev[0])

    def checkForDevice(self):
        return (True if self._GetAttached() else False)

    def waitfordevice(self, wait_time=70):
        found = False
        attempts = 0
        wait_period = 3
        while not found and (attempts*wait_period) < wait_time:
            if self.checkForDevice():
                found = True
            else:
                time.sleep(wait_period)                
                attempts += 1
                print attempts
            qApp.processEvents()

    def _mounts(self, name):
        try:           
            mtd,mount,fs,opt,x,y,r_rw = '','','','','','',''
            busybox = self._HasBusybox()
            if busybox:
                a = filter(None, Adb(self.adb)._shell('busybox realpath {}'.format(name)).strip().split('/'))[0]
            else:
                a = filter(None, Adb(self.adb)._shell('realpath {}'.format(name)).strip().split('/'))[0]
            if not a.startswith('/'):
                try:
                    mode = self._GetAttached()[1]
                    if mode in ['recovery', 'fastboot']:
                        mtd,mount,fs,opt,x,y = Adb(self.adb)._shell('cat /system/etc/fstab | grep {}'.format('/'+a)).strip().split(' ')
                    else:
                        mtd,mount,fs,opt,x,y = Adb(self.adb)._shell('cat /proc/mounts | grep {}'.format('/'+a)).strip().split(' ')                
                    if ('failed' or 'can\'t read') in mount:                        
                        return False
                    if mode in ['offline']:
                        return False                
                except ValueError:
                    return False
                r_rw = opt.split(',')[0]
        except IndexError:
            return False
        else:
            return mtd,mount,fs,opt,x,y,r_rw
            
    def _rwStatus(self, name):
        m = self._mounts(name)
        if m and len(m) == 7:
            mtd,mount,fs,opt,x,y,r_rw = m
            return r_rw
        else:
            return 'n/a'
            
    def _remount(self, name):
        m = self._mounts(name)
        if m and len(m) == 7:
            mtd,mount,fs,opt,x,y,r_rw = m
            if r_rw == 'ro':
                Adb(self.adb)._shell('mount -o remount,rw {0} {1}'.format(mtd, mount))
            if r_rw == 'ro':
                Adb(self.adb)._shell('su -c \"mount -o remount,rw {0} {1}\"'.format(mtd, mount))
            return (True if r_rw == 'rw' else False)
        else:
            return False            

    def _HasRootAccess(self):
        a = Adb(self.adb)._shell('id').strip().split(' ')
        return (True if 'root' in (a[0] and a[1]) else False)

    def _GetRootVersion(self):
        su = re.findall('.+', Adb(self.adb)._shell('su -v'))
        return ('' if not su else su[0])

    def _HasBusybox(self):
        return (True if len(re.findall('.+', Adb(self.adb)._shell('which busybox'))) > 0 else False)

    def _GetBusyboxVersion(self):
        if self._HasBusybox():
            return re.findall('BusyBox\s(.+)\s\(', Adb(self.adb)._shell('busybox --help'))[0]
        return ''

    def _HasPushed(self, src, out):
        if not src:
            return False
        output = Adb(self.adb, 'out')._push(src, out)
        if not output:
            return False
        if filter(lambda x: 'bytes in' in x, output.splitlines()):
            return str(output)
        return False

    def _HasPulled(self, src, out):
        if not src:
            return False
        output = Adb(self.adb, 'out')._pull(src, out)
        if not output:
            return False
        if filter(lambda x: 'bytes in' in x, output.splitlines()):
            return str(output)
        return False

    def _HasSuccess(self, src):
        return (True if re.search('Success', src) else False)
