#!/usr/bin/python3
# -*- coding: utf-8 -*-

'''
    Copyright (C) 2013 ElTonno @ Android-Hilfe.de

    PyX3 is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyX3 is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
'''

__title__ = 'Backup via ADB'
__version__ = 'rev 20160417'
__author__ = 'ElTonno @ Android-Hilfe.de'
__copyright__ = 'Copyright (C) 2014-2016'

try:
    import os
    import sys
    import re
    import time
    import posixpath
    from PyQt4.QtCore import (Qt, QSize, QProcess)
    from PyQt4.QtGui import (QMainWindow, QFont, QSizePolicy, QLabel,
                             QLineEdit, QPushButton, QComboBox, QVBoxLayout,
                             QHBoxLayout, QSpacerItem, QWidget, qApp,
                             QMessageBox, QFileDialog, QApplication)
except ImportError as e:
    print("[!] Required module missing. %s" % e.args[0])
    sys.exit(-1) 

try:
    reload  # Python 2.7
except NameError:
    try:
        from importlib import reload  # Python 3.4+
    except ImportError:
        from imp import reload  # Python 3.0 - 3.3
        
#sys.setdefaultencoding('UTF-8')

# DEFAULTS -------------------------------------------------------------------------------------------------------------------
if sys.platform == 'linux':
    OS = 'lin'
elif sys.platform == 'linux2':
    OS = 'lin'    
elif sys.platform.startswith('win'):
    OS = 'win'
elif sys.platform == 'darwin':
    OS = 'mac'
else:
    OS = 'default'

if OS == 'win':
    adbName = 'adb.exe'
    p = os.path
elif OS == 'lin':
    adbName = 'adb'
    p = posixpath

folderpath = '/home/gunnar/Arbeitsfläche/PyX3/Backup'
adbpath = '/home/gunnar/adt/sdk/platform-tools'

# MAINWINDOW -----------------------------------------------------------------------------------------------------------------

class MyBackupUI(QMainWindow):

    def __init__(self):
        super(MyBackupUI, self).__init__()
        self.initUI()

    def initUI(self):
        self.setWindowTitle(__title__)
        self.setStyleSheet('font: 8pt; font-family: Curier;')
        # Font
        font_head = QFont()
        font_head.setPointSize(12)
        font_head.setBold(True)
        font_head.setWeight(75)
        font_smaller = QFont()
        font_smaller.setPointSize(9)
        font_smaller.setBold(False)
        font_smaller.setWeight(75)
        font_normal = QFont()
        font_normal.setBold(True)
        font_normal.setWeight(75)
        # Geometry
        self.resize(340, 354)
        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
        self.setSizePolicy(sizePolicy)
        self.setMaximumSize(QSize(340, 354))
        # Labels
        self.lblHead = QLabel("<font color='darkred'>Backups only work on devices running 4.0+ !</font>")
        self.lblHead.setFont(font_head)
        self.lblHead.setAlignment(Qt.AlignCenter)
        self.lblWarning = QLabel("<font color='darkred'>( may not work for all devices )</font>")
        self.lblWarning.setFont(font_smaller)
        self.lblWarning.setAlignment(Qt.AlignCenter)
        self.lblBackupFolder = QLabel('Backup folder:')
        self.lblBackupFolder.setFont(font_normal)
        self.lblBackupFolder.setAlignment(Qt.AlignCenter)
        self.lblADBFolder = QLabel('ADB-Binary folder:')
        self.lblADBFolder.setFont(font_normal)
        self.lblADBFolder.setAlignment(Qt.AlignCenter)
        # Edit
        self.lineEdit = QLineEdit()
        self.lineEdit2 = QLineEdit()
        # Buttons
        self.btnBrowse = QPushButton('Browse')
        self.btnBrowse.clicked.connect(self.slotBrowse)
        self.btnBrowse.setMaximumSize(QSize(70, 25))
        self.btnBackup1 = QPushButton('Backup all without system apps')
        self.btnBackup1.clicked.connect(self.slotBackup1)
        self.btnBackup2 = QPushButton('Backup all with system apps (unsafe)')
        self.btnBackup2.clicked.connect(self.slotBackup2)
        self.btnBackup3 = QPushButton('Backup app and device data (not the APKs themselves)')
        self.btnBackup3.clicked.connect(self.slotBackup3)
        self.btnBackup4 = QPushButton('Backup apps')
        self.btnBackup4.clicked.connect(self.slotBackup4)
        self.btnBackup5 = QPushButton('Backup device shared storage / SD card contents')
        self.btnBackup5.clicked.connect(self.slotBackup5)
        self.btnRestore = QPushButton('Restore...')
        self.btnRestore.clicked.connect(self.slotRestore)
        self.btnBrowseADB = QPushButton('Browse')
        self.btnBrowseADB.clicked.connect(self.slotBrowseADB)
        self.btnBrowseADB.setMaximumSize(QSize(70, 25))
        self.btnRefresh = QPushButton('Refresh')
        self.btnRefresh.setMaximumSize(QSize(70, 25))
        self.btnRefresh.clicked.connect(self.getAttachedDevices)
        # ComboBox
        self.comboBox = QComboBox()
        # Layout
        self.vLayout = QVBoxLayout()
        self.vLayout.setSpacing(6)
        self.vLayout.addWidget(self.lblHead)
        self.vLayout.addWidget(self.lblWarning)
        spacerItem = QSpacerItem(20, 30, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.vLayout.addItem(spacerItem)
        self.vLayout.addWidget(self.lblBackupFolder)
        self.hLayout = QHBoxLayout()
        self.hLayout.addWidget(self.lineEdit)
        self.hLayout.addWidget(self.btnBrowse)
        self.vLayout.addLayout(self.hLayout)
        self.vLayout.addWidget(self.btnBackup1)
        self.vLayout.addWidget(self.btnBackup2)
        self.vLayout.addWidget(self.btnBackup3)
        self.vLayout.addWidget(self.btnBackup4)
        self.vLayout.addWidget(self.btnBackup5)
        spacerItem1 = QSpacerItem(20, 20, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.vLayout.addItem(spacerItem1)
        self.vLayout.addWidget(self.btnRestore)
        spacerItem2 = QSpacerItem(20, 20, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.vLayout.addItem(spacerItem2)
        self.vLayout.addWidget(self.lblADBFolder)
        self.hLayout2 = QHBoxLayout()
        self.hLayout2.addWidget(self.lineEdit2)
        self.hLayout2.addWidget(self.btnBrowseADB)
        self.vLayout.addLayout(self.hLayout2)
        self.hLayout2 = QHBoxLayout()
        self.hLayout2.addWidget(self.comboBox)
        self.hLayout2.addWidget(self.btnRefresh)
        self.vLayout.addLayout(self.hLayout2)
        # set MainLayout
        self.centralwidget = QWidget(self)
        self.setCentralWidget(self.centralwidget)
        self.vLayout2 = QVBoxLayout(self.centralwidget)
        self.vLayout2.addLayout(self.vLayout)
        # QProcess
        self.qprocess = QProcess()
        self.qprocess.setProcessChannelMode(QProcess.MergedChannels)
        # set defaults
        self.folderPath = folderpath
        self.adbPath = adbpath
        if self.folderPath:
            self.adbPath = p.join(adbpath, adbName)
        self.lineEdit.setText(folderpath)
        self.lineEdit2.setText(adbpath)
        self.globalSerial = ''
        self.getAttachedDevices()

    # QProcess ---------------------------------------------------------------------------------------------------------------
    def MyProcess(self, cmd):
        # correct root adb -------------
        if os.stat(self.adbPath).st_uid == 0:
            cmd = 'gksudo '+cmd
        # start process
        qproc_output = ''
        qproc = QProcess()
        qproc.setProcessChannelMode(QProcess.MergedChannels)
        qproc.start(cmd)
        qproc.waitForStarted()
        while qproc.state() == QProcess.Running:
            qApp.processEvents()
            qproc_output += str(qproc.readAll())
            time.sleep(0.002)    
        return qproc_output

    def MyProcessWait(self, cmd):
        self.qprocess.start(cmd)
        self.qprocess.waitForStarted()
        self.qprocess.waitForFinished()
        return self.qprocess.readAll()

    def killProcess(self):
        self.qprocess.terminate()

    # Check for Devices ------------------------------------------------------------------------------------------------------
    def getAttachedDevices(self):
        if not self.adbPath:
            return
        self.devices = []
        dev = re.findall("\\\\n(.+)\\\\tdevice", self.MyProcess(u'{0} devices'.format(self.adbPath)))
        if dev:     
            for item in dev:                
                self.devices.append(item)
        print(self.devices)
        if not self.devices:
            return
        if 'error' in self.devices:
            return
        self.comboBox.clear()
        self.comboBox.addItems(self.devices)
        self.comboBox.currentIndexChanged.connect(self.devicesChanged)
        self.devicesChanged()

    def devicesChanged(self):
        self.globalSerial = self.comboBox.currentText()

    # Check QLineEdit if empty -----------------------------------------------------------------------------------------------
    def checkEmpty(self):
        text1 = self.lineEdit.text()
        text2 = self.lineEdit2.text()
        ret = False
        if not text1 and not text2:
            msg = 'Please define the directories!'
            ret = True
        elif not text1:
            msg = 'Please define the Backup-Directory!'
            ret = True
        elif not text2:
            msg = 'Please define the ADB-Directory!'
            ret = True
        else:
            ret = False
        if ret:
            QMessageBox.information(self, 'Missing...', msg)
        return ret

    # Check for device -------------------------------------------------------------------------------------------------------
    def checkDevice(self):
        ret = False
        if not self.globalSerial:
            QMessageBox.information(self, 'Missing...', 'Please define a device!')
            ret = False
        else:
            ret = True
        return ret

    # SLOTS ------------------------------------------------------------------------------------------------------------------
    def slotBackup1(self):
        self.backup_new(1)

    def slotBackup2(self):
        self.backup_new(2)

    def slotBackup3(self):
        self.backup_new(3)

    def slotBackup4(self):
        self.backup_new(4)

    def slotBackup5(self):
        self.backup_new(5)

    def slotBrowse(self):
        try:
            f = QFileDialog.getExistingDirectory(self, 'Select Directory')
            if not f:
                return
            if p.isdir(f):
                self.folderPath = f
                self.lineEdit.setText(f)
            else:
                QMessageBox.information(self, 'Missing', 'Folder not found!')
        except IOError as e:
            QMessageBox.information(self, 'Browse...', e)
            return

    def slotRestore(self):
        try:
            f = QFileDialog.getOpenFileName(self,
                                                'Open File',
                                                'Open backup file',
                                                self.tr('Backup (*.ab)'))
            if not f:
                return
            if p.getsize(f) == 0:
                QMessageBox().information(self, 'Restore...', '{}, Filesize is zero'.format(p.basename(f)))
                return
            if p.isfile(f):
                self.backup_new(6, f)
            else:
                QMessageBox.information(self, 'Missing', 'File not found!')
                return
        except IOError as e:
            QMessageBox.information(self, 'Browse...', e)
            return

    def slotBrowseADB(self):
        try:
            f = QFileDialog.getExistingDirectory(self, 'Select Directory')
            if not f:
                return
            if p.isfile(p.join(f, adbName)):
                self.lineEdit2.setText(f)
                self.adbPath = p.join(f, adbName)
                self.getAttachedDevices()
            else:
                QMessageBox.information(self, 'Missing', 'Binary not found!')
                return
        except IOError as e:
            QMessageBox.information(self, 'Browse...', e)
            return

    # Backup-Funtion ---------------------------------------------------------------------------------------------------------
    def backup_new(self, value=0, backup_file=''):
        self.lineEdit.setEnabled(False)
        self.btnBrowse.setEnabled(False)
        self.btnBackup1.setEnabled(False)
        self.btnBackup2.setEnabled(False)
        self.btnBackup3.setEnabled(False)
        self.btnBackup4.setEnabled(False)
        self.btnBackup5.setEnabled(False)
        self.btnRestore.setEnabled(False)
        self.lineEdit2.setEnabled(False)
        self.btnBrowseADB.setEnabled(False)
        self.comboBox.setEnabled(False)
        self.btnRefresh.setEnabled(False)
        self.killProcess()
        try:
            if self.checkEmpty():
                return
            if not self.checkDevice():
                return
            try:
                sdk_version = self.MyProcess(u'{0} -s {1} shell getprop ro.build.version.sdk'.format(self.adbPath, self.globalSerial))
                if not int(sdk_version) >= 14:
                    QMessageBox().information(self, 'Backup...',
                                                'Your SDK-Version is not supported: {0}\nSupported SDK-Version: 14+ (Ice Cream Sandwich or newer)'.format(sdk_version))
                    return
            except ValueError:
                ret = QMessageBox.information(self, 'Get SDK-Version...', 'Unknown SDK-Version, continue?', '&Ok', '&Cancel', '', 0, 1)
                if ret == 1:
                    return
                pass

            path = self.folderPath
            if not p.isdir(path):
                os.mkdir(path)
            if value in [1, 2, 3, 4, 5]:
                QMessageBox().information(self, 'Backup...', 'Now unlock your Android-Screen!')
                if value == 1:
                    fname = 'without_system_apps.ab'
                    fname_path = p.join(path, fname)
                    self.MyProcess(u'{0} -s {1} backup -apk -shared -all -nosystem -f \"{2}\"'.format(self.adbPath, self.globalSerial, fname_path))
                elif value == 2:
                    fname = 'with_system_apps.ab'
                    fname_path = p.join(path, fname)
                    self.MyProcess(u'{0} -s {1} backup -apk -shared -all -system -f \"{2}\"'.format(self.adbPath, self.globalSerial, fname_path))
                elif value == 3:
                    fname = 'app_device_data.ab'
                    fname_path = p.join(path, fname)
                    self.MyProcess(u'{0} -s {1} backup -all -f \"{2}\"'.format(self.adbPath, self.globalSerial, fname_path))
                elif value == 4:
                    fname = 'apps.ab'
                    fname_path = p.join(path, fname)
                    self.MyProcess(u'{0} -s {1} backup -apk -noshared -nosystem -f \"{2}\"'.format(self.adbPath, self.globalSerial, fname_path))
                elif value == 5:
                    fname = 'storage.ab'
                    fname_path = p.join(path, fname)
                    self.MyProcess(u'{0} -s {1} backup -noapk -shared -nosystem -f \"{2}\"'.format(self.adbPath, self.globalSerial, fname_path))
                else:
                    return
                if p.isfile(fname_path):
                    if p.getsize(fname_path) == 0:
                        QMessageBox().information(self, 'Backup...', '{}, Filesize is zero'.format(fname))
                        os.remove(fname_path)
                        return
                    QMessageBox().information(self, 'Backup...', 'BACKUP: {}, Done'.format(fname))
                else:
                    QMessageBox().information(self, 'Backup...', 'BACKUP: {}, Failed'.format(fname))
            else:
                if value == 6:
                    QMessageBox().information(self, 'Restore...', 'Now unlock your Android-Screen!')
                    name, path = p.split(backup_file)
                    self.MyProcess(u'{0} -s {1} restore \"{2}\"'.format(self.adbPath, self.globalSerial, backup_file))
                    QMessageBox().information(self, 'Restore...', 'RESTORE: {}, Done'.format(name))
                else:
                    return
        finally:
            self.lineEdit.setEnabled(True)
            self.btnBrowse.setEnabled(True)
            self.btnBackup1.setEnabled(True)
            self.btnBackup2.setEnabled(True)
            self.btnBackup3.setEnabled(True)
            self.btnBackup4.setEnabled(True)
            self.btnBackup5.setEnabled(True)
            self.btnRestore.setEnabled(True)
            self.lineEdit2.setEnabled(True)
            self.btnBrowseADB.setEnabled(True)
            self.comboBox.setEnabled(True)
            self.btnRefresh.setEnabled(True)

    # closeEvent -------------------------------------------------------------------------------------------------------------
    def closeEvent(self, e):
        ret = QMessageBox.information(self, 'Quit...', 'Are you sure to quit?', '&Ok', '&Cancel', '', 0, 1)
        if ret == 0:
            self.killProcess()
            e.accept()
        else:
            e.ignore()


def main():
    app = QApplication(sys.argv)
    w = MyBackupUI()
    w.show()
    sys.exit(app.exec_())

if __name__ == '__main__':
    main()
